// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package it.esuli;

import it.esuli.ShakeListener.OnShakeListener;
import android.app.Activity;
import android.content.Intent;
import android.hardware.SensorManager;
import android.media.AudioManager;
import android.media.SoundPool;
import android.media.SoundPool.OnLoadCompleteListener;
import android.net.Uri;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.view.View;
import android.widget.SeekBar;
import android.widget.ToggleButton;

public class RingingBellsActivity extends Activity implements OnShakeListener {

	// shake
	private ShakeListener shakeListener;
	private float minShakeSpeed = 11;

	// audio
	private boolean playASingleBell;
	private static final int maxStreams = 10;
	private SoundPool soundPool;
	boolean soundLoaded;
	private int bellID;

	// power
	private WakeLock wakeLock;

	// UI
	private SeekBar pitchBar;
	private ToggleButton singleBellButton;
	private ToggleButton manyBellsButton;

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		// shake
		shakeListener = new ShakeListener(this);
		shakeListener.setOnShakeListener(this);

		// audio
		playASingleBell = false;
		if (savedInstanceState != null) {
			playASingleBell = savedInstanceState.getBoolean("Single");
		}
		setVolumeControlStream(AudioManager.STREAM_MUSIC);
		soundPool = new SoundPool(maxStreams, AudioManager.STREAM_MUSIC, 0);
		soundLoaded = false;
		soundPool.setOnLoadCompleteListener(new OnLoadCompleteListener() {
			public void onLoadComplete(SoundPool soundPool, int sampleId,
					int status) {
				soundLoaded = true;
			}
		});
		bellID = soundPool.load(this, R.raw.church_bell, 1);

		// power
		PowerManager powerManager = (PowerManager) getSystemService(POWER_SERVICE);
		wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK
				| PowerManager.ON_AFTER_RELEASE, getString(R.string.app_name));
		wakeLock.setReferenceCounted(false);

		// UI
		pitchBar = (SeekBar) findViewById(R.id.pitchBar);
		pitchBar.setMax(100);
		if (savedInstanceState != null)
			pitchBar.setProgress(savedInstanceState.getInt("Pitch"));
		else
			pitchBar.setProgress(50);
		pitchBar.setEnabled(playASingleBell);

		singleBellButton = (ToggleButton) findViewById(R.id.buttonSingleBell);
		manyBellsButton = (ToggleButton) findViewById(R.id.buttomManyBells);

		singleBellButton.setChecked(playASingleBell);
		manyBellsButton.setChecked(!playASingleBell);
	}

	public void onRestart() {
		super.onRestart();
	}

	public void onStart() {
		super.onStart();
	}

	public void onResume() {
		super.onResume();
		shakeListener.onResume();
	}

	public void singleBellClick(View v) {
		playASingleBell = true;
		pitchBar.setEnabled(true);
		singleBellButton.setChecked(playASingleBell);
		manyBellsButton.setChecked(!playASingleBell);
	}

	public void manyBellsClick(View v) {
		playASingleBell = false;
		pitchBar.setEnabled(false);
		singleBellButton.setChecked(playASingleBell);
		manyBellsButton.setChecked(!playASingleBell);
	}
	
	public void largeBellClick(View v) {
		if(pitchBar.isEnabled()) {
			int progress = pitchBar.getProgress();
			progress = Math.max(0, progress-10);
			pitchBar.setProgress(progress);
		}
	}

	public void smallBellClick(View v) {
		if(pitchBar.isEnabled()) {
			int progress = pitchBar.getProgress();
			progress = Math.min(100,progress+10);
			pitchBar.setProgress(progress);
		}
	}

	public void wwwClick(View v) {
		Intent browserIntent = new Intent(Intent.ACTION_VIEW,
				Uri.parse("http://www.esuli.it/"));
		startActivity(browserIntent);
	}

	public void playSound(int soundID, float rate) {
		if (soundLoaded) {
			AudioManager audioManager = (AudioManager) getSystemService(AUDIO_SERVICE);
			float actualVolume = (float) audioManager
					.getStreamVolume(AudioManager.STREAM_MUSIC);
			float maxVolume = (float) audioManager
					.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
			float volume = actualVolume / maxVolume;
			soundPool.play(soundID, volume, volume, 0, 0, rate);
		}
	}

	public void onPause() {
		super.onPause();

		shakeListener.onPause();

		soundPool.autoPause();
	}

	public void onStop() {
		super.onStop();
	}

	public void onDestroy() {
		super.onDestroy();
		soundPool.release();
	}

	public void onSaveInstanceState(Bundle savedInstanceState) {
		savedInstanceState.putBoolean("Single", playASingleBell);
		savedInstanceState.putInt("Pitch", pitchBar.getProgress());
		super.onSaveInstanceState(savedInstanceState);
	}

	public boolean isItAShake(float[] orientation, float[] acceleration) {
		return Math.abs(acceleration[SensorManager.DATA_X]) > minShakeSpeed;
	}

	public void onShake(float[] orientation, float[] acceleration) {
		if (playASingleBell) {
			float rate = ((pitchBar.getProgress() / 100.0f) * 1.5f) + 0.5f;
			playSound(bellID, rate);
		} else {
			float inclination = Math.abs(orientation[SensorManager.DATA_Z]) / 90f;
			inclination = (float) ((int) (inclination * 5.0f)) / 5.0f;
			float rate = Math.min(2.0f,
					Math.max(0.0f, inclination * 1.5f) + 0.5f);
			playSound(bellID, rate);
		}
		wakeLock.acquire(0);
	}
}